@@ -252,75 +252,72 @@ namespace CGAL {
 
 private:
     #ifndef CGAL_CFG_NO_CPP0X_VARIADIC_TEMPLATES
-    template <typename PrimitiveType, typename ... T>
+    template <typename ... T>
     void set_primitive_data_impl(CGAL::Boolean_tag<false>,T ... ){}
-    template <typename PrimitiveType, typename ... T>
+    template <typename ... T>
     void set_primitive_data_impl(CGAL::Boolean_tag<true>,T ... t)
-    {m_traits.template set_shared_data<PrimitiveType>(t...);}
+    {m_traits.set_shared_data(t...);}
 
-    template <typename PrimitiveType, typename ... T>
+    template <typename ... T>
     void set_shared_data(T...t){
-      set_primitive_data_impl<PrimitiveType>(CGAL::Boolean_tag<internal::Has_nested_type_Shared_data<PrimitiveType>::value>(),t...);
+      set_primitive_data_impl(CGAL::Boolean_tag<internal::Has_nested_type_Shared_data<Primitive>::value>(),t...);
     }
     #else
-    template <typename PrimitiveType> 
     void set_primitive_data_impl(CGAL::Boolean_tag<false>){}
-    template <typename PrimitiveType>
     void set_primitive_data_impl(CGAL::Boolean_tag<true>)
-    {m_traits.template set_shared_data<PrimitiveType>();}
-    template <typename PrimitiveType>
+    {m_traits.set_shared_data();}
     void set_shared_data(){
-      set_primitive_data_impl<PrimitiveType>(CGAL::Boolean_tag<internal::Has_nested_type_Shared_data<PrimitiveType>::value>());
+      set_primitive_data_impl(CGAL::Boolean_tag<internal::Has_nested_type_Shared_data<Primitive>::value>());
     }
     
-    template <typename PrimitiveType, typename T1> 
+    template <typename T1>
     void set_primitive_data_impl(CGAL::Boolean_tag<false>,T1){}
-    template <typename PrimitiveType, typename T1>
+    template <typename T1>
     void set_primitive_data_impl(CGAL::Boolean_tag<true>,T1 t1)
-    {m_traits.template set_shared_data<PrimitiveType>(t1);}
-    template <typename PrimitiveType, typename T1>
+    {m_traits.set_shared_data(t1);}
+    template <typename T1>
     void set_shared_data(T1 t1){
-      set_primitive_data_impl<PrimitiveType>(Boolean_tag<internal::Has_nested_type_Shared_data<PrimitiveType>::value>(),t1);
+      set_primitive_data_impl(Boolean_tag<internal::Has_nested_type_Shared_data<Primitive>::value>(),t1);
     }
     
-    template <typename PrimitiveType, typename T1, typename T2> 
+    template <typename T1, typename T2>
     void set_primitive_data_impl(CGAL::Boolean_tag<false>,T1,T2){}
-    template <typename PrimitiveType, typename T1, typename T2>
+    template <typename T1, typename T2>
     void set_primitive_data_impl(CGAL::Boolean_tag<true>,T1 t1,T2 t2)
-    {m_traits.template set_shared_data<PrimitiveType>(t1,t2);}
-    template <typename PrimitiveType, typename T1, typename T2>
+    {m_traits.set_shared_data(t1,t2);}
+    template <typename T1, typename T2>
     void set_shared_data(T1 t1,T2 t2){
-      set_primitive_data_impl<PrimitiveType>(Boolean_tag<internal::Has_nested_type_Shared_data<PrimitiveType>::value>(),t1,t2);
+      set_primitive_data_impl(Boolean_tag<internal::Has_nested_type_Shared_data<Primitive>::value>(),t1,t2);
     }
     
-    template <typename PrimitiveType, typename T1, typename T2, typename T3> 
+    template <typename T1, typename T2, typename T3>
     void set_primitive_data_impl(CGAL::Boolean_tag<false>,T1,T2,T3){}
-    template <typename PrimitiveType, typename T1, typename T2, typename T3>
+    template <typename T1, typename T2, typename T3>
     void set_primitive_data_impl(CGAL::Boolean_tag<true>,T1 t1,T2 t2,T3 t3)
-    {m_traits.template set_shared_data<PrimitiveType>(t1,t2,t3);}
-    template <typename PrimitiveType, typename T1, typename T2, typename T3>
+    {m_traits.set_shared_data(t1,t2,t3);}
+    template <typename T1, typename T2, typename T3>
     void set_shared_data(T1 t1,T2 t2,T3 t3){
-      set_primitive_data_impl<PrimitiveType>(Boolean_tag<internal::Has_nested_type_Shared_data<PrimitiveType>::value>(),t1,t2,t3);
+      set_primitive_data_impl(Boolean_tag<internal::Has_nested_type_Shared_data<Primitive>::value>(),t1,t2,t3);
     }
     
-    template <typename PrimitiveType, typename T1, typename T2, typename T3, typename T4> 
+    template <typename T1, typename T2, typename T3, typename T4>
     void set_primitive_data_impl(CGAL::Boolean_tag<false>,T1,T2,T3,T4){}
-    template <typename PrimitiveType, typename T1, typename T2, typename T3, typename T4>
+    template <typename T1, typename T2, typename T3, typename T4>
     void set_primitive_data_impl(CGAL::Boolean_tag<true>,T1 t1,T2 t2,T3 t3,T4 t4)
-    {m_traits.template set_shared_data<PrimitiveType>(t1,t2,t3,t4);}
-    template <typename PrimitiveType, typename T1, typename T2, typename T3, typename T4>
+    {m_traits.set_shared_data(t1,t2,t3,t4);}
+    template <typename T1, typename T2, typename T3, typename T4>
     void set_shared_data(T1 t1,T2 t2,T3 t3,T4 t4){
-      set_primitive_data_impl<PrimitiveType>(Boolean_tag<internal::Has_nested_type_Shared_data<PrimitiveType>::value>(),t1,t2,t3,t4);
+      set_primitive_data_impl(Boolean_tag<internal::Has_nested_type_Shared_data<Primitive>::value>(),t1,t2,t3,t4);
     }
     
-    template <typename PrimitiveType, typename T1, typename T2, typename T3, typename T4, typename T5> 
+    template <typename T1, typename T2, typename T3, typename T4, typename T5>
     void set_primitive_data_impl(CGAL::Boolean_tag<false>,T1,T2,T3,T4,T5){}
-    template <typename PrimitiveType, typename T1, typename T2, typename T3, typename T4, typename T5>
+    template <typename T1, typename T2, typename T3, typename T4, typename T5>
     void set_primitive_data_impl(CGAL::Boolean_tag<true>,T1 t1,T2 t2,T3 t3,T4 t4,T5 t5)
-    {m_traits.template set_shared_data<PrimitiveType>(t1,t2,t3,t4,t5);}
-    template <typename PrimitiveType, typename T1, typename T2, typename T3, typename T4, typename T5>
+    {m_traits.set_shared_data(t1,t2,t3,t4,t5);}
+    template <typename T1, typename T2, typename T3, typename T4, typename T5>
     void set_shared_data(T1 t1,T2 t2,T3 t3,T4 t4,T5 t5){
-      set_primitive_data_impl<PrimitiveType>(Boolean_tag<internal::Has_nested_type_Shared_data<PrimitiveType>::value>(),t1,t2,t3,t4,t5);
+      set_primitive_data_impl(Boolean_tag<internal::Has_nested_type_Shared_data<Primitive>::value>(),t1,t2,t3,t4,t5);
     }
     #endif
 
@@ -675,7 +672,7 @@ public:
                              ConstPrimitiveIterator beyond,
                              T ... t)
 	{
-    set_shared_data<Primitive>(t...);
+    set_shared_data(t...);
 		while(first != beyond)
 		{
 			m_primitives.push_back(Primitive(first,t...));
@@ -807,7 +804,7 @@ public:
 	void AABB_tree<Tr>::insert(ConstPrimitiveIterator first,
                              ConstPrimitiveIterator beyond)
 	{
-    set_shared_data<Primitive>();
+    set_shared_data();
 		while(first != beyond)
 		{
 			m_primitives.push_back(Primitive(first));
@@ -822,7 +819,7 @@ public:
                              ConstPrimitiveIterator beyond,
                              T1 t1)
 	{
-    set_shared_data<Primitive>(t1);
+    set_shared_data(t1);
 		while(first != beyond)
 		{
 			m_primitives.push_back(Primitive(first,t1));
@@ -837,7 +834,7 @@ public:
                              ConstPrimitiveIterator beyond,
                              T1 t1,T2 t2)
 	{
-    set_shared_data<Primitive>(t1,t2);
+    set_shared_data(t1,t2);
 		while(first != beyond)
 		{
 			m_primitives.push_back(Primitive(first,t1,t2));
@@ -852,7 +849,7 @@ public:
                              ConstPrimitiveIterator beyond,
                              T1 t1,T2 t2,T3 t3)
 	{
-    set_shared_data<Primitive>(t1,t2,t3);
+    set_shared_data(t1,t2,t3);
 		while(first != beyond)
 		{
 			m_primitives.push_back(Primitive(first,t1,t2,t3));
@@ -867,7 +864,7 @@ public:
                              ConstPrimitiveIterator beyond,
                              T1 t1,T2 t2,T3 t3,T4 t4)
 	{
-    set_shared_data<Primitive>(t1,t2,t3,t4);
+    set_shared_data(t1,t2,t3,t4);
 		while(first != beyond)
 		{
 			m_primitives.push_back(Primitive(first,t1,t2,t3,t4));
@@ -882,7 +879,7 @@ public:
                              ConstPrimitiveIterator beyond,
                              T1 t1,T2 t2,T3 t3,T4 t4,T5 t5)
 	{
-    set_shared_data<Primitive>(t1,t2,t3,t4,t5);
+    set_shared_data(t1,t2,t3,t4,t5);
 		while(first != beyond)
 		{
 			m_primitives.push_back(Primitive(first,t1,t2,t3,t4,t5));
