@@ -24,15 +24,16 @@ namespace CGAL {
 namespace Surface_mesh_simplification 
 {
 
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::EdgeCollapse( ECM&                    aSurface
-                                                    , ShouldStop       const& aShould_stop
-                                                    , VertexIndexMap   const& aVertex_index_map 
-                                                    , EdgeIndexMap     const& aEdge_index_map 
-                                                    , EdgeIsBorderMap  const& aEdge_is_border_map 
-                                                    , GetCost          const& aGet_cost
-                                                    , GetPlacement     const& aGet_placement
-                                                    , VisitorT                aVisitor
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::EdgeCollapse( ECM&                         aSurface
+                                                    , ShouldStop            const& aShould_stop
+                                                    , VertexIndexMap        const& aVertex_index_map
+                                                    , EdgeIndexMap          const& aEdge_index_map
+                                                    , EdgeIsBorderMap       const& aEdge_is_border_map
+                                                    , EdgeIsConstrainedMap  const& aEdge_is_constrained_map
+                                                    , GetCost               const& aGet_cost
+                                                    , GetPlacement          const& aGet_placement
+                                                    , VisitorT                     aVisitor
                                                     )
   : 
    mSurface           (aSurface)
@@ -40,6 +41,7 @@ EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::EdgeCollapse( ECM&                    aS
   ,Vertex_index_map   (aVertex_index_map)
   ,Edge_index_map     (aEdge_index_map)
   ,Edge_is_border_map (aEdge_is_border_map)
+  ,Edge_is_constrained_map (aEdge_is_constrained_map)
   ,Get_cost           (aGet_cost)
   ,Get_placement      (aGet_placement)
   ,Visitor            (aVisitor)
@@ -63,8 +65,8 @@ EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::EdgeCollapse( ECM&                    aS
 #endif
 }
 
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-int EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::run()
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+int EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::run()
 {
   CGAL_SURF_SIMPL_TEST_assertion( mSurface.is_valid() && mSurface.is_pure_triangle() ) ;
 
@@ -85,8 +87,8 @@ int EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::run()
   return r ;
 }
 
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Collect()
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+void EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::Collect()
 {
   CGAL_ECMS_TRACE(0,"Collecting edges...");
 
@@ -114,10 +116,12 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Collect()
   std::set<edge_descriptor> zero_length_edges;
 
   undirected_edge_iterator eb, ee ;
-  for ( boost::tie(eb,ee) = undirected_edges(mSurface); eb!=ee; ++eb )
+  for ( boost::tie(eb,ee) = undirected_edges(mSurface); eb!=ee; ++eb, id+=2 )
   {
     edge_descriptor lEdge = *eb ;
-  
+
+    if ( is_constrained(lEdge) ) continue;//no not insert constrainted edges
+
     CGAL_assertion( get_directed_edge_id(lEdge) == id ) ;
     CGAL_assertion( get_directed_edge_id(opposite_edge(lEdge,mSurface)) == id+1 ) ;
 
@@ -141,8 +145,6 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Collect()
 
     
     CGAL_ECMS_TRACE(2,edge_to_string(lEdge));
-    
-    id += 2 ;
   }
  
   CGAL_SURF_SIMPL_TEST_assertion ( lInserted + lNotInserted == mInitialEdgeCount ) ;
@@ -156,20 +158,48 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Collect()
 
     // edges of length 0 removed no longer need to be treated
     if ( lProfile.left_face_exists() )
-        zero_length_edges.erase( primary_edge(lProfile.vL_v0()) );
-    if  ( lProfile.right_face_exists() )
-        zero_length_edges.erase( primary_edge(lProfile.vR_v1()) );
+    {
+      edge_descriptor lEdge_to_remove = is_constrained(lProfile.vL_v0()) ?
+                                          primary_edge(lProfile.v1_vL()) :
+                                          primary_edge(lProfile.vL_v0()) ;
+      zero_length_edges.erase( lEdge_to_remove );
+      Edge_data& lData = get_data(lEdge_to_remove) ;
+      if ( lData.is_in_PQ() ){
+        CGAL_ECMS_TRACE(2,"Removing E" << get(Edge_index_map,lEdge_to_remove) << " from PQ" );
+        remove_from_PQ(lEdge_to_remove,lData);
+      }
+      --mCurrentEdgeCount;
+    }
+
+    if ( lProfile.right_face_exists() )
+    {
+      edge_descriptor lEdge_to_remove = is_constrained(lProfile.vR_v1()) ?
+                                          primary_edge(lProfile.v0_vR()) :
+                                          primary_edge(lProfile.vR_v1()) ;
+      zero_length_edges.erase( lEdge_to_remove );
+      Edge_data& lData = get_data(lEdge_to_remove) ;
+      if ( lData.is_in_PQ() ){
+        CGAL_ECMS_TRACE(2,"Removing E" << get(Edge_index_map,lEdge_to_remove) << " from PQ" );
+        remove_from_PQ(lEdge_to_remove,lData);
+      }
+      --mCurrentEdgeCount;
+    }
+
+    --mCurrentEdgeCount;
 
     //the placement is trivial, it's always the point itself
     Placement_type lPlacement = lProfile.p0();
-    Collapse(lProfile ,lPlacement);
+    vertex_descriptor rResult
+      = halfedge_collapse_bk_compatibility(lProfile.v0_v1(), Edge_is_constrained_map);
+    put(vertex_point,mSurface,rResult,*lPlacement);
+    Visitor.OnCollapsed(lProfile,rResult);
   }
-  
+
   CGAL_ECMS_TRACE(0,"Initial edge count: " << mInitialEdgeCount ) ;
 }
 
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Loop()
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+void EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::Loop()
 {
   CGAL_ECMS_TRACE(0,"Collapsing edges...") ;
 
@@ -180,18 +210,23 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Loop()
   // Pops and processes each edge from the PQ
   //
   optional<edge_descriptor> lEdge ;
+  #ifdef CGAL_SURF_SIMPL_INTERMEDIATE_STEPS_PRINTING
+  int i_rm=0;
+  #endif
   while ( (lEdge = pop_from_PQ()) )
   {
     CGAL_SURF_SIMPL_TEST_assertion ( lLoop_watchdog ++ < mInitialEdgeCount ) ;
 
     CGAL_ECMS_TRACE(1,"Poped " << edge_to_string(*lEdge) ) ;
+
+    CGAL_assertion( !is_constrained(*lEdge) );
     
     Profile const& lProfile = create_profile(*lEdge);
-      
+
     Cost_type lCost = get_data(*lEdge).cost();
     
     Visitor.OnSelected(lProfile,lCost,mInitialEdgeCount,mCurrentEdgeCount);
-      
+
     if ( lCost ) 
     {
       if ( Should_stop(*lCost,lProfile,mInitialEdgeCount,mCurrentEdgeCount) )
@@ -212,7 +247,22 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Loop()
         Placement_type lPlacement = get_placement(lProfile);
         
         if ( Is_collapse_geometrically_valid(lProfile,lPlacement) )
+        {
+          #ifdef CGAL_SURF_SIMPL_INTERMEDIATE_STEPS_PRINTING
+          std::cout << "step " << i_rm << " " << source(*lEdge,mSurface)->point() << " " << target(*lEdge,mSurface)->point() << "\n";
+          #endif
           Collapse(lProfile,lPlacement);
+          #ifdef CGAL_SURF_SIMPL_INTERMEDIATE_STEPS_PRINTING
+          std::stringstream sstr;
+          sstr << "debug/P-";
+          if (i_rm<10) sstr << "0";
+          if (i_rm<100) sstr << "0";
+          sstr <<  i_rm  << ".off";
+          std::ofstream out(sstr.str().c_str());
+          out << mSurface;
+          ++i_rm;
+          #endif
+        }
       }
       else
       {
@@ -230,8 +280,8 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Loop()
   }
 }
 
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::is_border( const_vertex_descriptor const& aV ) const
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+bool EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::is_border( const_vertex_descriptor const& aV ) const
 {
   bool rR = false ;
   
@@ -249,22 +299,44 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::is_border( const_vertex_descriptor
   return rR ;  
 }
 
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+bool EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::is_border_or_constrained( const_vertex_descriptor const& aV ) const
+{
+  const_in_edge_iterator eb, ee ;
+  for ( boost::tie(eb,ee) = in_edges(aV,mSurface) ; eb != ee ; ++ eb )
+  {
+    const_edge_descriptor lEdge = *eb ;
+    if ( is_undirected_edge_a_border(lEdge) || is_constrained(lEdge) )
+      return true;
+  }
+  return false;
+}
+
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+bool EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::is_constrained( const_vertex_descriptor const& aV ) const
+{
+  const_in_edge_iterator eb, ee ;
+  for ( boost::tie(eb,ee) = in_edges(aV,mSurface) ; eb != ee ; ++ eb )
+    if ( is_constrained(*eb) ) return true;
+  return false;
+}
+
 // Some edges are NOT collapsable: doing so would break the topological consistency of the mesh.
 // This function returns true if a edge 'p->q' can be collapsed.
 //
 // An edge p->q can be collapsed iff it satisfies the "link condition"
 // (as described in the "Mesh Optimization" article of Hoppe et al (1993))
 //
-// The link conidition is as follows: for every vertex 'k' adjacent to both 'p and 'q',
+// The link condition is as follows: for every vertex 'k' adjacent to both 'p and 'q',
 // "p,k,q" is a facet of the mesh.
 //
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_topologically_valid( Profile const& aProfile )
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+bool EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::Is_collapse_topologically_valid( Profile const& aProfile )
 {
   bool rR = true ;
 
-  CGAL_ECMS_TRACE(3,"Testing topological collapsabilty of p_q=V" << aProfile.v0()->id() << "(%" << aProfile.v0()->vertex_degree() << ")"
-                 << "->V" << aProfile.v1()->id() << "(%" << aProfile.v1()->vertex_degree() << ")" 
+  CGAL_ECMS_TRACE(3,"Testing topological collapsabilty of p_q=V" << get(Vertex_index_map,aProfile.v0()) << "(%" << aProfile.v0()->vertex_degree() << ")"
+                 << "->V" << get(Vertex_index_map,aProfile.v1()) << "(%" << aProfile.v1()->vertex_degree() << ")" 
                  );
                  
   CGAL_ECMS_TRACE(4, "is p_q border:" << aProfile.is_v0_v1_a_border() );
@@ -274,13 +346,13 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_topologically_valid( Pr
   out_edge_iterator eb2, ee2 ; 
 
   CGAL_ECMS_TRACE(4,"  t=V" 
-                   << ( aProfile.left_face_exists() ? aProfile.vL()->id() : -1 )
+                   << ( aProfile.left_face_exists() ? get(Vertex_index_map,aProfile.vL()) : -1 )
                    << "(%" 
                    << ( aProfile.left_face_exists() ? aProfile.vL()->vertex_degree() : 0 ) 
                    << ")" 
                  );
   CGAL_ECMS_TRACE(4,"  b=V" 
-                   << ( aProfile.right_face_exists() ? aProfile.vR()->id() : -1 )
+                   << ( aProfile.right_face_exists() ? get(Vertex_index_map,aProfile.vR()) : -1 )
                    << "(%" 
                    << ( aProfile.right_face_exists() ? aProfile.vR()->vertex_degree() :0 ) 
                    << ")" 
@@ -343,13 +415,13 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_topologically_valid( Pr
 
           if ( !lIsFace )
           {
-            CGAL_ECMS_TRACE(3,"  k=V" << k->id() << " IS NOT in a face with p-q. NON-COLLAPSABLE edge." ) ;
+            CGAL_ECMS_TRACE(3,"  k=V" << get(Vertex_index_map,k) << " IS NOT in a face with p-q. NON-COLLAPSABLE edge." ) ;
             rR = false ;
             break ;
           }  
           else 
           {
-            CGAL_ECMS_TRACE(4,"  k=V" << k->id() << " is in a face with p-q") ;
+            CGAL_ECMS_TRACE(4,"  k=V" << get(Vertex_index_map,k) << " is in a face with p-q") ;
           }
         }
       }  
@@ -358,6 +430,10 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_topologically_valid( Pr
      
   if ( rR )
   {
+    /// ensure two constrained edges cannot get merged
+    if ( is_edge_adjacent_to_a_constrained_edge(
+          aProfile, Edge_is_constrained_map) ) return false ;
+
     if ( aProfile.is_v0_v1_a_border() )
     {
       if ( Is_open_triangle(aProfile.v0_v1()) )
@@ -399,8 +475,8 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_topologically_valid( Pr
   return rR ;
 }
 
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_tetrahedron( edge_descriptor const& h1 )
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+bool EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::Is_tetrahedron( edge_descriptor const& h1 )
 {
   //
   // Code copied from Polyhedron_3::is_tetrahedron()
@@ -451,8 +527,8 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_tetrahedron( edge_descriptor con
   return true;
 }
 
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_open_triangle( edge_descriptor const& h1 )
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+bool EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::Is_open_triangle( edge_descriptor const& h1 )
 {
   bool rR = false ;
   
@@ -463,7 +539,7 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_open_triangle( edge_descriptor c
   if ( next_edge(h3,mSurface) == h1 )
   {
     // Now check if it is open
-    CGAL_ECMS_TRACE(4,"  p-q is a border edge... checking E" << h2->id() << " and E" << h3->id() ) ;
+    CGAL_ECMS_TRACE(4,"  p-q is a border edge... checking E" << get(Edge_index_map,h2) << " and E" << get(Edge_index_map,h3) ) ;
     
     rR = is_border(h2) && is_border(h3);  
   
@@ -482,8 +558,8 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_open_triangle( edge_descriptor c
 // respective areas is no greater than a max value and the internal
 // dihedral angle formed by their supporting planes is no greater than
 // a given threshold
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::are_shared_triangles_valid( Point const& p0, Point const& p1, Point const& p2, Point const& p3 ) const 
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+bool EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::are_shared_triangles_valid( Point const& p0, Point const& p1, Point const& p2, Point const& p3 ) const
 {
   bool rR = false ;
   
@@ -533,9 +609,9 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::are_shared_triangles_valid( Point c
 }
 
 // Returns the directed halfedge connecting v0 to v1, if exists.
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-typename EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::edge_descriptor
-EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::find_connection ( const_vertex_descriptor const& v0, const_vertex_descriptor const& v1 ) const
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+typename EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::edge_descriptor
+EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::find_connection ( const_vertex_descriptor const& v0, const_vertex_descriptor const& v1 ) const
 {
   out_edge_iterator eb, ee ; 
   for ( boost::tie(eb,ee) = out_edges(v0,mSurface) ; eb != ee ; ++ eb )
@@ -549,10 +625,10 @@ EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::find_connection ( const_vertex_descripto
 }
 
 // Given the edge 'e' around the link for the collapsinge edge "v0-v1", finds the vertex that makes a triangle adjacent to 'e' but exterior to the link (i.e not containing v0 nor v1)
-// If 'e' is a mnull handle OR 'e' is a border edge, there is no such triangle and a null handle is returned.
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-typename EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::vertex_descriptor
-EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::find_exterior_link_triangle_3rd_vertex ( const_edge_descriptor const& e, const_vertex_descriptor const& v0, const_vertex_descriptor const& v1 ) const
+// If 'e' is a null handle OR 'e' is a border edge, there is no such triangle and a null handle is returned.
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+typename EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::vertex_descriptor
+EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::find_exterior_link_triangle_3rd_vertex ( const_edge_descriptor const& e, const_vertex_descriptor const& v0, const_vertex_descriptor const& v1 ) const
 {
   vertex_descriptor r ;
   
@@ -581,15 +657,15 @@ EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::find_exterior_link_triangle_3rd_vertex (
 }
 
 
-// A collase is geometrically valid if, in the resulting local mesh no two adjacent triangles form an internal dihedral angle
+// A collapse is geometrically valid if, in the resulting local mesh no two adjacent triangles form an internal dihedral angle
 // greater than a fixed threshold (i.e. triangles do not "fold" into each other)
 //
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_geometrically_valid( Profile const& aProfile, Placement_type k0)
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+bool EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::Is_collapse_geometrically_valid( Profile const& aProfile, Placement_type k0)
 {
   bool rR = true ;
   
-  CGAL_ECMS_TRACE(3,"Testing geometrical collapsabilty of v0-v1=E" << aProfile.v0_v1()->id() );
+  CGAL_ECMS_TRACE(3,"Testing geometrical collapsabilty of v0-v1=E" << get(Edge_index_map,aProfile.v0_v1()) );
   if ( k0 )
   {
     //
@@ -610,7 +686,7 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_geometrically_valid( Pr
       vertex_descriptor k2 = * l ;
       vertex_descriptor k3 = *nx ;
       
-      CGAL_ECMS_TRACE(4,"  Screening link vertices k1=V" << k1->id() << " k2=V" << k2->id() << " k3=V" << k3->id() ) ;
+      CGAL_ECMS_TRACE(4,"  Screening link vertices k1=V" << get(Vertex_index_map,k1) << " k2=V" << get(Vertex_index_map,k2) << " k3=V" << get(Vertex_index_map,k3) ) ;
       
       edge_descriptor e12 = find_connection(k1,k2);
       edge_descriptor e23 = k3 != k1 ? find_connection(k2,k3) : edge_descriptor() ;
@@ -622,7 +698,7 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_geometrically_valid( Pr
         
         if ( !are_shared_triangles_valid( *k0, get_point(k1), get_point(k2), get_point(k3) ) )
         {
-          CGAL_ECMS_TRACE(3,"    Triangles VX-V" << k1->id() << "-V" << k2->id() << " and VX-V" << k3->id() << " are not geometrically valid. Collapse rejected");
+          CGAL_ECMS_TRACE(3,"    Triangles VX-V" << get(Vertex_index_map,k1) << "-V" << get(Vertex_index_map,k2) << " and VX-V" << get(Vertex_index_map,k3) << " are not geometrically valid. Collapse rejected");
           rR = false ;
         }
       }
@@ -635,11 +711,11 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_geometrically_valid( Pr
         // There is indeed a triangle shared along e12
         if ( handle_assigned(k4) )
         {
-          CGAL_ECMS_TRACE(4,"    Found exterior link triangle shared along E" << e12->id() << " with third vertex: V" << k4->id() ) ;
+          CGAL_ECMS_TRACE(4,"    Found exterior link triangle shared along E" << get(Edge_index_map,e12) << " with third vertex: V" << get(Vertex_index_map,k4) ) ;
           
           if ( !are_shared_triangles_valid( get_point(k1), get_point(k4), get_point(k2), *k0 ) )
           {
-            CGAL_ECMS_TRACE(3,"    Triangles V" << k1->id() << "-V" << k4->id() << " and V" << k2->id() << "-VX are not geometrically valid. Collapse rejected");
+            CGAL_ECMS_TRACE(3,"    Triangles V" << get(Vertex_index_map,k1) << "-V" << get(Vertex_index_map,k4) << " and V" << get(Vertex_index_map,k2) << "-VX are not geometrically valid. Collapse rejected");
             rR = false ;
           }
         }
@@ -653,11 +729,11 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_geometrically_valid( Pr
         // There is indeed a triangle shared along e12
         if ( handle_assigned(k5) )
         {
-          CGAL_ECMS_TRACE(4,"    Found exterior link triangle shared along E" << e23->id() << " with third vertex: V" << k5->id() ) ;
+          CGAL_ECMS_TRACE(4,"    Found exterior link triangle shared along E" << get(Edge_index_map,e23) << " with third vertex: V" << get(Vertex_index_map,k5) ) ;
           
           if ( !are_shared_triangles_valid( get_point(k2), get_point(k5), get_point(k3), *k0 ) )
           {
-            CGAL_ECMS_TRACE(3,"    Triangles V" << k2->id() << "-V" << k5->id() << " and V" << k3->id() << "-VX are not geometrically valid. Collapse rejected");
+            CGAL_ECMS_TRACE(3,"    Triangles V" << get(Vertex_index_map,k2) << "-V" << get(Vertex_index_map,k5) << " and V" << get(Vertex_index_map,k3) << "-VX are not geometrically valid. Collapse rejected");
             rR = false ;
           }
         }
@@ -669,8 +745,8 @@ bool EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Is_collapse_geometrically_valid( Pr
   return rR ;
 }
 
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Collapse( Profile const& aProfile, Placement_type aPlacement )
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+void EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::Collapse( Profile const& aProfile, Placement_type aPlacement )
 {
 
   CGAL_ECMS_TRACE(1,"S" << mStep << ". Collapsig " << edge_to_string(aProfile.v0_v1()) ) ;
@@ -691,15 +767,17 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Collapse( Profile const& aProfile,
   if ( aProfile.left_face_exists() )
   {
     edge_descriptor lV0VL = primary_edge(aProfile.vL_v0());
+    if ( is_constrained(lV0VL) ) //make sure a constrained edge will not disappear
+      lV0VL=primary_edge(aProfile.v1_vL());
     
-    CGAL_ECMS_TRACE(3,"V0VL E" << lV0VL->id()
-                   << "(V" <<  lV0VL->vertex()->id() << "->V" << lV0VL->opposite()->vertex()->id() << ")"
+    CGAL_ECMS_TRACE(3,"V0VL E" << get(Edge_index_map,lV0VL)
+                   << "(V" <<  get(Vertex_index_map,lV0VL->vertex()) << "->V" << get(Vertex_index_map,lV0VL->opposite()->vertex()) << ")"
                    ) ;
                    
     Edge_data& lData = get_data(lV0VL) ;
     if ( lData.is_in_PQ() )
     {
-      CGAL_ECMS_TRACE(2,"Removing E" << lV0VL->id() << " from PQ" ) ;
+      CGAL_ECMS_TRACE(2,"Removing E" << get(Edge_index_map,lV0VL) << " from PQ" ) ;
       remove_from_PQ(lV0VL,lData) ;
     }
 
@@ -710,22 +788,24 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Collapse( Profile const& aProfile,
   if ( aProfile.right_face_exists() )
   {
     edge_descriptor lVRV1 = primary_edge(aProfile.vR_v1());
-    
-    CGAL_ECMS_TRACE(3,"V1VRE" << lVRV1->id()
-                   << "(V" <<  lVRV1->vertex()->id() << "->V" << lVRV1->opposite()->vertex()->id() << ")"
+    if ( is_constrained(lVRV1) ) //make sure a constrained edge will not disappear
+      lVRV1=primary_edge(aProfile.v0_vR());
+
+    CGAL_ECMS_TRACE(3,"V1VRE" << get(Edge_index_map,lVRV1)
+                   << "(V" <<  get(Vertex_index_map,lVRV1->vertex()) << "->V" << get(Vertex_index_map,lVRV1->opposite()->vertex()) << ")"
                    ) ;
                    
     Edge_data& lData = get_data(lVRV1) ;
     if ( lData.is_in_PQ() )
     {
-      CGAL_ECMS_TRACE(2,"Removing E" << lVRV1->id() << " from PQ") ;
+      CGAL_ECMS_TRACE(2,"Removing E" << get(Edge_index_map,lVRV1) << " from PQ") ;
       remove_from_PQ(lVRV1,lData) ;
     }
     -- mCurrentEdgeCount ;
     CGAL_SURF_SIMPL_TEST_assertion_code( -- lResultingEdgeCount ) ; 
   }
 
-  CGAL_ECMS_TRACE(1,"Removing:\n  v0v1: E" << aProfile.v0_v1()->id() << "(V" << aProfile.v0()->id() << "->V" << aProfile.v1()->id() << ")" );
+  CGAL_ECMS_TRACE(1,"Removing:\n  v0v1: E" << get(Edge_index_map,aProfile.v0_v1()) << "(V" << get(Vertex_index_map,aProfile.v0()) << "->V" << get(Vertex_index_map,aProfile.v1()) << ")" );
   
     
   // Perform the actuall collapse.
@@ -734,7 +814,7 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Collapse( Profile const& aProfile,
   // (PT and QB are removed if they are not null).
   // All other edges must be kept.
   // All directed edges incident to vertex removed are relink to the vertex kept.
-  rResult = halfedge_collapse(aProfile.v0_v1(),mSurface);
+  rResult = halfedge_collapse_bk_compatibility(aProfile.v0_v1(), Edge_is_constrained_map);
 
   CGAL_SURF_SIMPL_TEST_assertion_code( -- lResultingEdgeCount ) ; 
 
@@ -746,7 +826,7 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Collapse( Profile const& aProfile,
 
   CGAL_SURF_SIMPL_TEST_assertion( mSurface.is_valid() && mSurface.is_pure_triangle() ) ;
   
-  CGAL_ECMS_TRACE(1,"V" << rResult->id() << " kept." ) ;
+  CGAL_ECMS_TRACE(1,"V" << get(Vertex_index_map,rResult) << " kept." ) ;
                  
 #ifdef CGAL_SURFACE_SIMPLIFICATION_ENABLE_TRACE
   out_edge_iterator eb1, ee1 ;     
@@ -767,8 +847,8 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Collapse( Profile const& aProfile,
   CGAL_ECMS_DEBUG_CODE ( ++mStep ; )
 }
 
-template<class M,class SP, class VIM,class EIM,class EBM, class CF,class PF,class V>
-void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Update_neighbors( vertex_descriptor const& aKeptV )
+template<class M,class SP, class VIM,class EIM,class EBM,class ECTM, class CF,class PF,class V>
+void EdgeCollapse<M,SP,VIM,EIM,EBM,ECTM,CF,PF,V>::Update_neighbors( vertex_descriptor const& aKeptV )
 {
   CGAL_ECMS_TRACE(3,"Updating cost of neighboring edges..." ) ;
 
@@ -779,6 +859,7 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Update_neighbors( vertex_descriptor
   typedef std::set<edge_descriptor,Compare_id> edges ;
   
   edges lToUpdate(Compare_id(this)) ;
+  edges lToInsert(Compare_id(this)) ;
   
   // (A.1) Loop around all vertices adjacent to the vertex kept
   in_edge_iterator eb1, ee1 ; 
@@ -795,12 +876,14 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Update_neighbors( vertex_descriptor
       edge_descriptor lEdge2 = primary_edge(*eb2) ;
       
       Edge_data& lData2 = get_data(lEdge2);
-      CGAL_ECMS_TRACE(4,"Inedge around V" << lAdj_k->id() << edge_to_string(lEdge2) ) ;
+      CGAL_ECMS_TRACE(4,"Inedge around V" << get(Vertex_index_map,lAdj_k) << edge_to_string(lEdge2) ) ;
     
-      // Only those edges still in the PQ _and_ not already collected are updated.
-      if ( lData2.is_in_PQ() && lToUpdate.find(lEdge2) == lToUpdate.end() )
+      // Only edges still in the PQ needs to be updated, the other needs to be re-inserted
+      if ( lData2.is_in_PQ() )
         lToUpdate.insert(lEdge2) ;
-    } 
+      else
+        lToInsert.insert(lEdge2) ;
+    }
   }  
   
   //
@@ -821,7 +904,28 @@ void EdgeCollapse<M,SP,VIM,EIM,EBM,CF,PF,V>::Update_neighbors( vertex_descriptor
     
     update_in_PQ(lEdge,lData);
   }
-    
+
+  //
+  // (C) Insert ignored edges
+  //
+  // I think that this should be done for edges eliminated because of the geometric criteria
+  // and not the topological one.However maintaining such a set might be more expensive
+  // and hard to be safe ...
+  for ( typename edges::iterator it = lToInsert.begin(),
+                                 eit = lToInsert.end() ; it != eit ; ++ it )
+  {
+    edge_descriptor lEdge = *it;
+    if ( is_constrained(lEdge) ) continue; //do not insert constrained edges
+    Edge_data& lData = get_data(lEdge);
+
+    Profile const& lProfile = create_profile(lEdge);
+
+    lData.cost() = get_cost(lProfile) ;
+
+    CGAL_ECMS_TRACE(3, edge_to_string(lEdge) << " re-inserted in the PQ") ;
+
+    insert_in_PQ(lEdge,lData);
+  }
 }
 
 
